#include <limits.h>  // needed for INT_MAX

/*
     This function merges the two (sorted) parts of the input array
     between indices p and r (inclusive) into a sorted array between
     indices p and r.

     The index q divides the input array into its two (sorted) parts.

     This function assumes the sub-array from p to q (inclusive) is sorted
     and the sub-array from q+1 to r (inclusive) is sorted.

     The two parts of the input array are not assumed to be of the same size.
*/
int merge(int* inputArray, int p, int q, int r)
{
   // pointers to the two temporary arrays needed for merging
   int* leftArray;
   int* rightArray;
   // dimensions for the two temporary arrays
   int  leftSize = q - p + 2;  // an extra space for the sentinel
   int rightSize = r - q + 1;  // an extra space for the sentinel

   int leftIndex, rightIndex;
   int i;  // loop variable

   // allocate the two temporary arrays
   leftArray  = (int*)calloc(leftSize, sizeof(int));
   if (leftArray == NULL)  // error check
   {
      fprintf(stderr, "merge: not enough memory for temporary array.\n");
      return -1;
   }
   rightArray = (int*)calloc(rightSize, sizeof(int));
   if (rightArray == NULL)  // error check
   {
      fprintf(stderr, "merge: not enough memory for temporary array.\n");
      return -1;
   }

   // copy the two parts of the input array into the two temporary arrays
   for (i = 0; i < leftSize-1; i++)
   {
      leftArray[i]  = inputArray[p+i];
   }
   for (i = 0; i < rightSize-1; i++)
   {
      rightArray[i] = inputArray[q+1+i];
   }

   // put the sentinal value at the end of each temporary array
   leftArray[leftSize-1]   = INT_MAX;
   rightArray[rightSize-1] = INT_MAX;

   // merge values from the two temporary arrays back into the input array
   leftIndex = 0;
   rightIndex = 0;
   for (i = p; i <= r; i++)
   {
      if (leftArray[leftIndex] <= rightArray[rightIndex])
      {
         inputArray[i] = leftArray[leftIndex];
         leftIndex++;
      }
      else
      {
         inputArray[i] = rightArray[rightIndex];
         rightIndex++;
      }
   }

   // deallocate the temporary arrays
   free(leftArray);
   free(rightArray);

   return 0;
}//merge()



/*
    This function uses the recursive merge sort algorithm to sort
    that part of the input array between indices p and q (inclusive).
*/
void mergeSort(int* inputArray, int p, int q)
{
   int cutoff; //index separating the left and right parts of the input array

   // check for a trivial array or incorrect indices
   if ( q-p < 1 ) return;

   cutoff = (p+q-1)/2; // when the input array has odd length, the right part is larger

   // recursively sort the array
   mergeSort(inputArray, p, cutoff);   // sort the left part
   mergeSort(inputArray, cutoff+1, q); // sort the right part
   merge(inputArray, p, cutoff, q);    // merge the two sorted parts together

}//mergeSort()
