
fun f 0 = "zero"
 |  f _ = "non-zero";

 fun f n =
   if n = 0 then
     "zero"
   else
     "non-zero";


fun fact n =
  if n = 0 then
    1
  else
    n * fact n-1;
    
fun fact 0 = 1
 |  fact n = n * fact n-1;
 
 
fun reverse L =
  if null L then
    nil
  else
    reverse(tl L) @ [hd L];
    
fun reverse nil = nil
 |  reverse (first :: rest) = reverse rest @ [first];
 
 
 (* sum a list of numbers *)
 fun f nil = 0
  |  f (first :: rest) = first + f rest;
  
  
(* increment each number in a list of numbers *)
fun f nil = nil
 |  f (first :: rest) = first + 1 :: f rest;
 
 
 (* how we define the scope of a local variable *)
 let
   val x = 1
   val y = 2
 in
   x = y
 end;


fun days2ms days =
  let
    val hours = days * 24
    val minutes = hours * 60
    val seconds = minutes * 60
  in
    seconds * 1000
  end;
  


(* the next three functions define merge sort *)

fun halve nil = (nil, nil)
 |  halve [a] = ([a], nil)
 |  halve (a::b::cs) =
      let
        val (x,y) = halve cs
      in
        (a::x, b::y)
      end;
      
      

fun merge (nil, ys) = ys
 |  merge (xs, nil) = xs
 |  merge (x::xs, y::ys) =
      if (x < y) then
        x::merge(xs, y::ys)
      else
        y::merge(x::xs, ys);
        
        

fun mergeSort nil = nil
 |  mergeSort [a] = [a]
 |  mergeSort theList =
      let
        val (x,y) = halve theList
      in
        merge( mergeSort x, mergeSort y )
      end;
      
      

(* the next two functions define quicksort *)

fun split(pivot, nil) = (nil,nil)
 |  split(pivot, x::xs) =
      let
        val (below, above) = split(pivot, xs)
      in
         if x < pivot then 
           (x::below, above)
         else 
           (below, x::above)
      end;  
      
fun quicksort nil = nil      
 |  quicksort (pivot::rest) =
      let
        val (below, above) = split(pivot, rest)
      in
        quicksort below @ [pivot] @ quicksort above
      end;
      
      
      
(* 
   Here is quicksort with split as a nested function.
   Notice that pivot no longer needs to be a parameter 
   to split. The pivot parameter of quickssort is now 
   within the scope of the nested version of split.
*)      

fun quicksortv2 nil = nil
 |  quicksortv2 (pivot::rest) =
      let
        fun split(nil) = (nil,nil)
         |  split(x::xs) =
              let
                val (below, above) = split(xs)
              in
                 if x<pivot then 
                   (x::below, above)
                 else 
                   (below, x::above)
              end;
        val (below, above) = split(rest)
      in
        quicksortv2 below @ [pivot] @ quicksortv2 above
      end;      